# !/usr/bin/perl
###################################################################################
#	Author: Russel Cornelson
#	Last Modified: 11/12/2015
#
#	Description:  This perl script can be used to parse a large file into a set of 
#				  smaller files.  Normal usage is to provide it a filepath, and
#				  the number of lines you want per output file.  You may optionally
#				  provide it a filename "test.txt" then declare a inbound path like
#				  C:\Folder\ or reroute output files to a different path.  You can 
#				  also optionally provide a string to break on, if you would like to
#				  create a new file each time a specific string is encountered.
#				
#	usage:  fp.pl <file> <numberoflines> <inboundPath> <outboundPath> <breakstring>
###################################################################################

BEGIN {
    unshift @INC, $1 if $0 =~ /^(.*)[\/\\]/;
}

##### PERL MODULE INCLUDES #####
use strict;
use File::Basename;
use File::Copy;
use File::Path;
use Getopt::Std;

##### VARIABLE DECLARATION #####
my $filename;  			#file name to parse into smaller pieces
my $path;				#file path to $filename
my $suffix;				#file extension for filename
my $inboundFile; 		#full path and file extension for incoming file
my $outboundFile;		#full path and file extension for outbound file
my $returnList;			#list of output files
my $lcount = 0;			#line counter
my $partnumber = 1;		#part number for outbound file pieces
my $stringmatch;

#####  HANDLE SCRIPT VARIABLES #####
$|=1;
use vars qw/$opt_f $opt_n $opt_i $opt_o $opt_h $opt_s/;

##### VALIDATE PARAMETERS #####
getopts('f:n:i:o:s:');
&Usage() if ($opt_h);
&Usage("Required Parameter missing: -f <filename>") unless $opt_f;
&Usage("Required Parameter missing: -n <output lines>") unless $opt_n;
&Usage("Provide Parameter -i <path> if Parameter -f is not a full path to file.") if (basename($opt_f) eq $opt_f) && (!$opt_i);
if (!$opt_s) {
	$stringmatch = "Thereisnowaythisstringexistsinlogfilethatiamparsingitisveryunlikely";
}
else {
	$stringmatch = $opt_s;
}


##### MAIN PROGRAM LOGIC #####


&ParseFilename;   																#parse the inbound file into path,name and suffix
$inboundFile = &BuildFullPath($path,$filename,$suffix);							#build the full path to inbound file
open (IN, $inboundFile) or die "Can not open '$inboundFile'";					#open the inbound file for reading


while (<IN>) {																	#read through each inbound file line
	if (($_ =~ /$stringmatch/) && (!$returnList eq "")) {						#if current line matches string AND list not empty
		close OUT;																#close output file
		$partnumber++;															#increment partnumber
		$lcount = 0;															#set count back to 0
	}
	if ($lcount eq 0) {															#if counter is 0;  open a new outbound File
		$outboundFile = &BuildFullPath($path,$filename,$suffix,$partnumber);	#get name of outbound file
		$returnList = $returnList . $outboundFile . ",";						#add outbound file name to a list the script will return
		open (OUT,">>",$outboundFile);											#open the outbound file
	}
	print OUT $_;																#print line from IN to OUT
	$lcount++;																	#add 1 to counter
	if ($lcount eq $opt_n) {													#if counter = MAX lines per output file, close and increment
		close OUT;
		$partnumber++;
		$lcount = 0;
	}
}	

close OUT;
close IN;

$returnList = substr($returnList,0,length($returnList)-1);
print $returnList;

##### END MAIN LOGIC #####


##### SUBROUTINES #####
sub BuildFullPath
{
	if ($#_ eq 2) {
		if ($opt_i) {
			return $opt_i . $filename . $suffix;
		}
		else {
			return $path . $filename . $suffix;
		}
	}
	elsif ($#_ eq 3) {
		if ($opt_o) {
			return $opt_o . $filename . $_[3] . $suffix; 
		}
		else {
			return $path . $filename . $_[3] . $suffix; 
		}
	}
	else {
		print "ERROR:  BuildFullPath must provide 3 or 4 parameters.";
		exit 1;
	}
	
}

sub ParseFilename
{
	if (basename($opt_f) eq $opt_f) {
		($filename,$path,$suffix) = fileparse($opt_f, '.[^\.]*');
		$path = $opt_i;
	}
	else {
		($filename,$path,$suffix) = fileparse($opt_f, '.[^\.]*');
	}
}


sub Usage
{
	my $msg = shift;
	print "\n\n$msg\n\n";
	print "\nUsage:\n";
	print "perl $0 -f -n [-i] [-o] [-s]\n\n";
	print "\t-f\tfile to parse: -f C:\\Folder\\filename.ext\n";
	print "\t-n\tmax output lines: -n 1000000\n";
	print "\t-i\toptional, inbound path: -i C:\\Folder\\\n";
	print "\t-o\toptional, outbound path: -o C:\\OutputFolder\\\n";
	print "\t-s\toptional, break on string match: -s string\n";
	exit 1;
}